@@ -24,7 +24,7 @@
 
 # region imports
 import argparse
-from re import findall
+from re import findall, sub
 from filecmp import cmp
 from os.path import join
 from pathlib import Path
@@ -35,7 +35,7 @@ from random import shuffle
 # region globals
 FILE_FORMAT = {'mp1', 'mp2', 'mp3', 'mp4', 'aac', 'ogg', 'wav', 'wma',
                'avi', 'xvid', 'divx', 'mpeg', 'mpg', 'mov', 'wmv'}
-__version__ = '1.0.1'
+__version__ = '1.2.0'
 
 
 # endregion
@@ -63,17 +63,25 @@ def get_args():
     parser.add_argument("-f", "--format", help="Select only a file format", type=str, choices=FILE_FORMAT)
     parser.add_argument("-z", "--size", help="Start size in bytes", type=int, default=1, metavar='BYTES')
     parser.add_argument("-m", "--max-tracks", help="Maximum number of tracks", type=int, default=None, metavar='NUMBER')
+    parser.add_argument("-t", "--title", help="Playlist title", default=None)
+    parser.add_argument("-g", "--encoding", help="Text encoding", choices=('UTF-8', 'ASCII', 'UNICODE'), default=None)
+    parser.add_argument("-I", "--image", help="Playlist image", default=None)
+    parser.add_argument("-l", "--link", help="Add remote file links", nargs=argparse.ONE_OR_MORE, default=[])
     parser.add_argument("-r", "--recursive", help="Recursive search", action='store_true')
     parser.add_argument("-a", "--absolute", help="Absolute file name", action='store_true')
     parser.add_argument("-s", "--shuffle", help="Casual order", action='store_true')
     parser.add_argument("-u", "--unique", help="The same files are not placed in the playlist", action='store_true')
     parser.add_argument("-c", "--append", help="Continue playlist instead of override it", action='store_true')
+    parser.add_argument("-w", "--windows", help="Windows style folder separator", action='store_true')
 
     args = parser.parse_args()
 
     # Check extension of playlist file
     if not args.playlist.endswith('.m3u'):
-        args.playlist += '.m3u'
+        if args.encoding == 'UNICODE':
+            args.playlist += '.m3u8'
+        else:
+            args.playlist += '.m3u'
 
     # Open playlist file
     mode = 'at' if args.append else 'wt'
@@ -93,6 +101,9 @@ def get_args():
 def file_in_playlist(playlist, file, root=None):
     """Check if file is in the playlist"""
     for f in playlist:
+        # Skip extended tags
+        if f.startswith('#'):
+            continue
         # Check if absolute path in playlist
         if root:
             f = join(root, f)
@@ -107,6 +118,27 @@ def main():
     args = get_args()
     multimedia_files = list()
 
+    # Check if playlist is an extended M3U
+    if args.title or args.encoding or args.image:
+        multimedia_files.insert(0, '#EXTM3U')
+        if args.max_tracks:
+            args.max_tracks += 1
+
+        # Set encoding
+        if args.encoding:
+            multimedia_files.insert(1, f'#EXTENC: {args.encoding}')
+            if args.max_tracks:
+                args.max_tracks += 1
+
+        # Set title
+        if args.title:
+            multimedia_files.append(f'#PLAYLIST: {args.title.capitalize()}')
+            if args.max_tracks:
+                args.max_tracks += 1
+
+    # Add link
+    multimedia_files.extend(args.link)
+
     # Walk to directories
     for directory in args.directories:
         # Build a Path object
@@ -132,7 +164,9 @@ def main():
                 if findall(args.pattern, file):
                     # Check file size
                     if size >= args.size:
-                        multimedia_files.append(file)
+                        multimedia_files.append(
+                            sub('/', r"\\", file) if args.windows else file
+                        )
 
     # Build a playlist
     if multimedia_files:
@@ -140,7 +174,8 @@ def main():
         if args.shuffle:
             shuffle(multimedia_files)
         with args.playlist as playlist:
-            playlist.writelines('\n'.join(multimedia_files[:args.max_tracks]))
+            joined_string = f'\n#EXTIMG: {args.image}\n' if args.image else '\n'
+            playlist.writelines(f'{joined_string}'.join(multimedia_files[:args.max_tracks]))
     else:
         print(f'warning: No multimedia files found here: {",".join(args.directories)}')
 
